@@ -4,7 +4,7 @@
 /// better performance.
 ///
 /// However if have to use serde for other reasons or are parsing
-/// directly to structs this is th4 place to go.
+/// directly to structs this is the place to go.
 ///
 mod de;
 mod se;
@@ -83,7 +83,7 @@ where
 ///
 /// # Errors
 ///
-/// Will return `Err` if an IO error is encountred while reading
+/// Will return `Err` if an IO error is encountered while reading
 /// rdr or if the readers content is invalid JSON.
 #[cfg_attr(not(feature = "no-inline"), inline(always))]
 pub fn from_reader<R, T>(mut rdr: R) -> Result<T>
@@ -395,44 +395,109 @@ impl<'value> TryInto<serde_json::Value> for BorrowedValue<'value> {
 #[cfg(test)]
 mod test {
     #![allow(clippy::unwrap_used)]
-    use crate::{json, BorrowedValue, OwnedValue};
-    use serde_json::{json as sjson, Value as SerdeValue};
+    use crate::{
+        error::Error, json, BorrowedValue, Deserializer as SimdDeserializer, ErrorType, OwnedValue,
+    };
+    use float_cmp::assert_approx_eq;
+    use halfbrown::{hashmap, HashMap};
+    use serde::{Deserialize, Serialize};
+    use serde_json::{json as sjson, to_string as sto_string, Value as SerdeValue};
+    use std::collections::BTreeMap;
     use std::convert::TryInto;
+
+    #[derive(Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
+    struct UnitStruct;
+    #[derive(Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
+    struct NewTypeStruct(u8);
+    #[derive(Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
+    struct TupleStruct(u8, u8);
+    #[derive(Debug, Serialize, Deserialize)]
+    struct TestStruct {
+        value: String,
+    }
+    #[derive(Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
+    struct TestStruct2 {
+        value: u8,
+    }
+    #[derive(Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
+    enum E {
+        NewTypeVariant(u8),
+        UnitVariant,
+        StructVariant { r: u8, g: u8, b: u8 },
+        StructVariant2 { r: u8, g: u8, b: u8 },
+        TupleVariant(u8, u8, u8),
+    }
+    #[derive(Debug, Serialize, Deserialize)]
+    struct TestPoint(f64, f64);
+
     #[test]
     fn convert_owned_value() {
         let v: OwnedValue = json!({
             "int": 42,
+            "int2": i64::MAX as u64 + 1,
             "float": 7.2,
             "neg-int": -23,
             "string": "string",
+            "bytes": b"bytes",
             "bool": true,
             "null": null,
+            "array": [42, 7, -23, false, null, {"key": "value"}],
             "object": {
             "array": [42, 7, -23, false, null, {"key": "value"}],
-            }
+            },
+            "tuple": (122, -14, true, 13_i8, -14_i16, 'c', 22_u8, 23_u16, 24_u32, 25_u64, (), None as Option<i32>, Some(3.25_f32), b"bytes"),
+            "struct": TestStruct{value: "value".to_string()},
+            "test_struct": TestStruct2{value: 3},
+            "point": TestPoint(3., 4.),
+            "unit_variant": E::UnitVariant,
+            "new_type_variant": E::NewTypeVariant(3),
+            "struct_variant": E::StructVariant{r:0, g:0, b:0},
+            "tuple_variant": E::TupleVariant(3, 4, 5),
         });
 
         let s: SerdeValue = sjson!({
             "int": 42,
+            "int2": i64::MAX as u64 + 1,
             "float": 7.2,
             "neg-int": -23,
             "string": "string",
+            "bytes": b"bytes",
             "bool": true,
             "null": null,
+            "array": [42, 7, -23, false, null, {"key": "value"}],
             "object": {
             "array": [42, 7, -23, false, null, {"key": "value"}],
-            }
+            },
+            "tuple": (122, -14, true, 13_i8, -14_i16, 'c', 22_u8, 23_u16, 24_u32, 25_u64, (), None as Option<i32>, Some(3.25_f32), b"bytes"),
+            "struct": TestStruct{value: "value".to_string()},
+            "test_struct": TestStruct2{value: 3},
+            "point": TestPoint(3., 4.),
+            "unit_variant": E::UnitVariant,
+            "new_type_variant": E::NewTypeVariant(3),
+            "struct_variant": E::StructVariant{r:0, g:0, b:0},
+            "tuple_variant": E::TupleVariant(3, 4, 5),
         });
         let s_c: SerdeValue = v.clone().try_into().unwrap();
         assert_eq!(s, s_c);
         let v_c: OwnedValue = s.try_into().unwrap();
         assert_eq!(v, v_c);
+
+        let mut v_ser = crate::serde::to_string(&v).unwrap();
+        let s_ser = serde_json::to_string(&v).unwrap();
+        assert_eq!(s_ser, v_ser);
+
+        let s_deser: OwnedValue = serde_json::from_str(&v_ser).unwrap();
+        assert_eq!(v, s_deser);
+
+        let v_deser: OwnedValue = crate::serde::from_str(&mut v_ser).unwrap();
+        assert_eq!(v, v_deser);
     }
 
     #[test]
     fn convert_borrowed_value() {
         let v: BorrowedValue = json!({
             "int": 42,
+            "int2": i64::MAX as u64 + 1,
             "float": 7.2,
             "neg-int": -23,
             "string": "string",
@@ -440,12 +505,24 @@ mod test {
             "null": null,
             "object": {
             "array": [42, 7, -23, false, null, {"key": "value"}],
-            }
+            },
+            "tuple": (122, -14, true, 13_i8, -14_i16, 'c', 22_u8, 23_u16, 24_u32, 25_u64, (), None as Option<i32>, Some(3.25_f32), b"bytes"),
+            "unit_struct": UnitStruct,
+            "new_type_struct": NewTypeStruct(3),
+            "tuple_struct": TupleStruct(3, 4),
+            "struct": TestStruct{value: "value".to_string()},
+            "test_struct": TestStruct2{value: 3},
+            "point": TestPoint(3., 4.),
+            "unit_variant": E::UnitVariant,
+            "new_type_variant": E::NewTypeVariant(3),
+            "struct_variant": E::StructVariant{r:0, g:0, b:0},
+            "tuple_variant": E::TupleVariant(3, 4, 5),
         })
         .into();
 
         let s: SerdeValue = sjson!({
             "int": 42,
+            "int2": i64::MAX as u64 + 1,
             "float": 7.2,
             "neg-int": -23,
             "string": "string",
@@ -453,7 +530,18 @@ mod test {
             "null": null,
             "object": {
             "array": [42, 7, -23, false, null, {"key": "value"}],
-            }
+            },
+            "tuple": (122, -14, true, 13_i8, -14_i16, 'c', 22_u8, 23_u16, 24_u32, 25_u64, (), None as Option<i32>, Some(3.25_f32), b"bytes"),
+            "unit_struct": UnitStruct,
+            "new_type_struct": NewTypeStruct(3),
+            "tuple_struct": TupleStruct(3, 4),
+            "struct": TestStruct{value: "value".to_string()},
+            "test_struct": TestStruct2{value: 3},
+            "point": TestPoint(3., 4.),
+            "unit_variant": E::UnitVariant,
+            "new_type_variant": E::NewTypeVariant(3),
+            "struct_variant": E::StructVariant{r:0, g:0, b:0},
+            "tuple_variant": E::TupleVariant(3, 4, 5),
         });
         let s_c: SerdeValue = v.clone().try_into().unwrap();
         assert_eq!(s, s_c);
@@ -488,19 +576,13 @@ mod test {
 
     #[test]
     fn convert_enum() {
+        #[allow(dead_code)]
         #[derive(serde::Deserialize, Debug)]
         #[serde(tag = "type")]
         enum Message {
             Request { id: usize, method: String },
             Response { id: String, result: String },
         }
-        let mut raw_json = r#"{"type": "Request", "id": 1, "method": "..."}"#.to_string();
-        let result: Result<Message, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
-        assert!(result.is_ok());
-
-        let mut raw_json = r#"{"type": "Response", "id": "1", "result": "..."}"#.to_string();
-        let result: Result<Message, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
-        assert!(result.is_ok());
 
         #[derive(serde::Deserialize, Debug)]
         #[serde(tag = "type", content = "v")]
@@ -509,13 +591,6 @@ mod test {
             Green { v: bool },
             Blue,
         }
-        let mut raw_json = r#"{"type": "Red", "v": "1"}"#.to_string();
-        let result: Result<Color, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
-        assert!(result.is_ok());
-
-        let mut raw_json = r#"{"type": "Blue"}"#.to_string();
-        let result: Result<Color, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
-        assert!(result.is_ok());
 
         #[derive(serde::Deserialize, Debug)]
         #[serde(tag = "type")]
@@ -524,6 +599,23 @@ mod test {
             Green { v: bool }, // TODO: If `content` flag is absent, `Green` works and `Red` doesn't
             Blue,
         }
+
+        let mut raw_json = r#"{"type": "Request", "id": 1, "method": "..."}"#.to_string();
+        let result: Result<Message, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
+        assert!(result.is_ok());
+
+        let mut raw_json = r#"{"type": "Response", "id": "1", "result": "..."}"#.to_string();
+        let result: Result<Message, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
+        assert!(result.is_ok());
+
+        let mut raw_json = r#"{"type": "Red", "v": "1"}"#.to_string();
+        let result: Result<Color, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
+        assert!(result.is_ok());
+
+        let mut raw_json = r#"{"type": "Blue"}"#.to_string();
+        let result: Result<Color, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
+        assert!(result.is_ok());
+
         let mut raw_json = r#"{"type": "Green", "v": false}"#.to_string();
         let result: Result<Color1, _> = super::from_slice(unsafe { raw_json.as_bytes_mut() });
         assert!(result.is_ok());
@@ -576,4 +668,231 @@ mod test {
         assert_eq!(p.x, 1);
         assert_eq!(p.y, 2);
     }
+
+    #[test]
+    #[allow(clippy::cast_precision_loss)]
+    fn floats() {
+        #[derive(serde_ext::Deserialize)]
+        struct Point {
+            x: f64,
+            y: f64,
+        }
+
+        let mut json = br#"{"x":1.0,"y":2.0}"#.to_vec();
+
+        let p: Point = crate::from_slice(&mut json).unwrap();
+        assert_approx_eq!(f64, p.x, 1_f64);
+        assert_approx_eq!(f64, p.y, 2_f64);
+
+        let json = json!({"x":-1,"y":i64::MAX as u64 + 1});
+
+        let p: Point = crate::from_str(&mut crate::to_string(&json).unwrap()).unwrap();
+        assert_approx_eq!(f64, p.x, -1_f64);
+        assert_approx_eq!(f64, p.y, i64::MAX as f64 + 1.0);
+    }
+
+    #[test]
+    fn vectors() {
+        let input: Vec<UnitStruct> = vec![UnitStruct];
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(
+            input,
+            crate::from_str::<Vec<UnitStruct>>(&mut v_str).unwrap()
+        );
+        let input: Vec<()> = Vec::new();
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(input, crate::from_str::<Vec<()>>(&mut v_str).unwrap());
+        let input: Vec<Option<u8>> = vec![None, Some(3_u8)];
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(
+            input,
+            crate::from_str::<Vec<Option<u8>>>(&mut v_str).unwrap()
+        );
+        let input: Vec<(i32, f32)> = vec![(3, 3.)];
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(
+            input,
+            crate::from_str::<Vec<(i32, f32)>>(&mut v_str).unwrap()
+        );
+        let input = vec![vec![3_u8]];
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(input, crate::from_str::<Vec<Vec<u8>>>(&mut v_str).unwrap());
+        let input: Vec<NewTypeStruct> = vec![NewTypeStruct(3_u8)];
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(
+            input,
+            crate::from_str::<Vec<NewTypeStruct>>(&mut v_str).unwrap()
+        );
+        let input: Vec<TupleStruct> = Vec::new();
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(
+            input,
+            crate::from_str::<Vec<TupleStruct>>(&mut v_str).unwrap()
+        );
+        let input = vec![TupleStruct(3, 3)];
+        let mut v_str = crate::to_string(&input).unwrap();
+        assert_eq!(
+            input,
+            crate::from_str::<Vec<TupleStruct>>(&mut v_str).unwrap()
+        );
+        let input = vec![E::NewTypeVariant(3)];
+        let mut _v_str = crate::to_string(&input).unwrap();
+        // Enums are not handled yet
+        // assert_eq!(input, crate::from_str::<Vec<E>>(&mut v_str).unwrap());
+        let input = vec![E::UnitVariant, E::UnitVariant];
+        let mut _v_str = crate::to_string(&input).unwrap();
+        // Enums are not handled yet
+        // assert_eq!(input, crate::from_str::<Vec<E>>(&mut v_str).unwrap());
+        let input = vec![
+            E::StructVariant { r: 0, g: 0, b: 0 },
+            E::StructVariant { r: 0, g: 0, b: 1 },
+        ];
+        let mut _v_str = crate::to_string(&input).unwrap();
+        // Enums are not handled yet
+        // assert_eq!(input, crate::from_str::<Vec<E>>(&mut v_str).unwrap());
+        let input = vec![E::TupleVariant(0, 0, 0), E::TupleVariant(1, 1, 1)];
+        let mut _v_str = crate::to_string(&input).unwrap();
+        // Enums are not handled yet
+        // assert_eq!(input, crate::from_str::<Vec<E>>(&mut v_str).unwrap());
+    }
+
+    macro_rules! parsing_error {
+        ($input:expr; $type:ty => $err:ident) => {{
+            let mut json_str = $input.to_string();
+            assert_eq!(
+                crate::from_str::<$type>(&mut json_str),
+                Err(SimdDeserializer::error(ErrorType::$err))
+            );
+        }};
+    }
+
+    #[test]
+    fn test_parsing_errors() {
+        parsing_error!(r#""3""#; i8 => ExpectedSigned);
+        parsing_error!(r#""3""#; i16 => ExpectedSigned);
+        parsing_error!(r#""3""#; i32 => ExpectedSigned);
+        parsing_error!(r#""3""#; i64 => ExpectedSigned);
+        parsing_error!(r#""3""#; u8 => ExpectedUnsigned);
+        parsing_error!(r#""3""#; u16 => ExpectedUnsigned);
+        parsing_error!(r#""3""#; u32 => ExpectedUnsigned);
+        parsing_error!(r#""3""#; u64 => ExpectedUnsigned);
+
+        parsing_error!("null"; i8 => ExpectedSigned);
+        parsing_error!("null"; i16 => ExpectedSigned);
+        parsing_error!("null"; i32 => ExpectedSigned);
+        parsing_error!("null"; i64 => ExpectedSigned);
+        parsing_error!("-3"; u8 => ExpectedUnsigned);
+        parsing_error!("-3"; u16 => ExpectedUnsigned);
+        parsing_error!("-3"; u32 => ExpectedUnsigned);
+        parsing_error!("-3"; u64 => ExpectedUnsigned);
+        parsing_error!("-3"; String => ExpectedString);
+
+        #[cfg(feature = "128bit")]
+        {
+            parsing_error!(r#""3""#; i128 => ExpectedSigned);
+            parsing_error!(r#""3""#; u128 => ExpectedUnsigned);
+            parsing_error!("null"; i128 => ExpectedSigned);
+            parsing_error!("-3"; u128 => ExpectedUnsigned);
+        }
+
+        parsing_error!("null"; f64 => ExpectedFloat);
+    }
+
+    macro_rules! ser_deser_map {
+        ($key:expr => $value:expr, $type:ty) => {
+            let input = hashmap! {$key => $value};
+            let mut m_str = crate::to_string(&input).unwrap();
+            assert_eq!(m_str, sto_string(&input).unwrap());
+            assert_eq!(input, crate::from_str::<$type>(&mut m_str).unwrap());
+        };
+    }
+
+    #[test]
+    fn maps() {
+        let key_error = Err(Error::generic(ErrorType::KeyMustBeAString));
+        assert_eq!(crate::to_string(&hashmap! {b"1234" => 3_i8}), key_error);
+        assert_eq!(crate::to_string(&hashmap! {true => 3_i8}), key_error);
+        assert_eq!(
+            crate::to_string(&hashmap! {[3_u8, 4_u8] => 3_i8}),
+            key_error
+        );
+        assert_eq!(
+            crate::to_string(&hashmap! {None as Option<u8> => 3_i8}),
+            key_error
+        );
+        assert_eq!(crate::to_string(&hashmap! {Some(3_u8) => 3_i8}), key_error);
+        assert_eq!(crate::to_string(&hashmap! {() => 3_i8}), key_error);
+        assert_eq!(crate::to_string(&hashmap! {(3, 4) => 3_i8}), key_error);
+        assert_eq!(crate::to_string(&hashmap! {[3, 4] => 3_i8}), key_error);
+        assert_eq!(crate::to_string(&hashmap! {UnitStruct => 3_i8}), key_error);
+        assert_eq!(
+            crate::to_string(&hashmap! {TupleStruct(3, 3) => 3_i8}),
+            key_error
+        );
+        assert_eq!(
+            crate::to_string(&hashmap! {TestStruct2{value:3} => 3_i8}),
+            key_error
+        );
+        assert_eq!(
+            crate::to_string(&hashmap! {E::NewTypeVariant(0) => 3_i8}),
+            key_error
+        );
+        assert_eq!(
+            crate::to_string(&hashmap! {E::StructVariant{r:0, g:0, b:0} => 3_i8}),
+            key_error
+        );
+        assert_eq!(
+            crate::to_string(&hashmap! {E::StructVariant2{r:0, g:0, b:0} => 3_i8}),
+            key_error
+        );
+        assert_eq!(
+            crate::to_string(&hashmap! {E::TupleVariant(0, 0, 0) => 3_i8}),
+            key_error
+        );
+        assert_eq!(
+            crate::to_string(&hashmap! {vec![0, 0, 0] => 3_i8}),
+            key_error
+        );
+        let mut m = BTreeMap::new();
+        m.insert("value", 3_u8);
+        assert_eq!(crate::to_string(&hashmap! {m => 3_i8}), key_error);
+
+        // f32 and f64 do not implement std::cmp:Eq nor Hash traits
+        // assert_eq!(crate::to_string(&hashmap! {3f32 => 3i8}), key_error);
+        // assert_eq!(crate::to_string(&hashmap! {3f64 => 3i8}), key_error);
+
+        let mut input = std::collections::HashMap::new();
+        input.insert(128_u8, "3");
+        let mut input_str = crate::to_string(&input).unwrap();
+        assert_eq!(input_str, sto_string(&input).unwrap());
+        assert_eq!(
+            crate::from_str::<std::collections::HashMap<u8, i8>>(&mut input_str),
+            Err(Error::new(0, '?', ErrorType::ExpectedSigned))
+        );
+        assert_eq!(
+            crate::from_str::<std::collections::HashMap<i8, String>>(&mut input_str),
+            Err(Error::new(0, '?', ErrorType::InvalidNumber))
+        );
+        assert_eq!(
+            crate::from_str::<HashMap<Option<u8>, String>>(&mut input_str),
+            Ok(hashmap! {Some(128_u8) => "3".to_string()})
+        );
+
+        ser_deser_map!('c' => 3_i8, HashMap<char, i8>);
+        ser_deser_map!(3_i8 => 3_i8, HashMap<i8, i8>);
+        ser_deser_map!(3_i16 => 3_i8, HashMap<i16, i8>);
+        ser_deser_map!(3_i32 => 3_i8, HashMap<i32, i8>);
+        ser_deser_map!(3_i64 => 3_i8, HashMap<i64, i8>);
+        ser_deser_map!(3_u8 => 3_i8, HashMap<u8, i8>);
+        ser_deser_map!(3_u16 => 3_i8, HashMap<u16, i8>);
+        ser_deser_map!(3_u32 => 3_i8, HashMap<u32, i8>);
+        ser_deser_map!(3_u64 => 3_i8, HashMap<u64, i8>);
+        #[cfg(feature = "128bit")]
+        {
+            ser_deser_map!(3_i128 => 3_i8, HashMap<i128, i8>);
+            ser_deser_map!(3_u128 => 3_i8, HashMap<u128, i8>);
+        }
+        ser_deser_map!(NewTypeStruct(1) => 3_i8, HashMap<NewTypeStruct, i8>);
+        ser_deser_map!(E::UnitVariant => 3_i8, HashMap<E, i8>);
+    }
 }
